import tempfile
import shutil
import os
import hmac
import base64
import hashlib
import urllib
import datetime

from webtest import TestApp

from quixote import cleanup, get_publisher
from quixote.wsgi import QWIP
from wcs import publisher
from wcs.qommon.http_request import HTTPRequest
from wcs.users import User
from wcs.categories import Category

pub, req, app_dir, user = None, None, None, None

QWIP.request_class = HTTPRequest

def setup_module(module):
    cleanup()

    global pub, req, app_dir, user
    APP_DIR = tempfile.mkdtemp()
    publisher.WcsPublisher.APP_DIR = APP_DIR
    pub = publisher.WcsPublisher.create_publisher()
    # allow saving the user
    pub.app_dir = os.path.join(APP_DIR, 'example.net')
    os.mkdir(pub.app_dir)
    user = User()
    user.name = 'Jean Darmette'
    user.email = 'jean.darmette@triffouilis.fr'
    user.store()

    file(os.path.join(pub.app_dir, 'site-options.cfg'), 'w').write('''\
[api-secrets]
coucou = 1234
''')

    category = Category()
    category.name = 'category'
    category.store()


def teardown_module(module):
    global pub
    shutil.rmtree(pub.APP_DIR)

def get_app():
    return TestApp(QWIP(pub), extra_environ={'HTTP_HOST': 'example.net'})

def test_user_page_redirect():
    output = get_app().get('/user')
    assert output.headers.get('location') == 'http://example.net/myspace/'

def test_user_page_error_when_json_and_no_user():
    output = get_app().get('/user?format=json')
    assert output.body == '???'

def test_get_user_from_api_query_string_error_missing_orig():
    output = get_app().get('/user?format=json&signature=xxx', status=403)
    assert output.json['err_desc'] == 'missing/multiple orig field'

def test_get_user_from_api_query_string_error_invalid_orig():
    output = get_app().get('/user?format=json&orig=coin&signature=xxx', status=403)
    assert output.json['err_desc'] == 'invalid orig'

def test_get_user_from_api_query_string_error_missing_algo():
    output = get_app().get('/user?format=json&orig=coucou&signature=xxx', status=403)
    assert output.json['err_desc'] == 'missing/multiple algo field'

def test_get_user_from_api_query_string_error_invalid_algo():
    output = get_app().get('/user?format=json&orig=coucou&signature=xxx&algo=coin', status=403)
    assert output.json['err_desc'] == 'invalid algo'

def test_get_user_from_api_query_string_error_invalid_signature():
    output = get_app().get('/user?format=json&orig=coucou&signature=xxx&algo=sha1', status=403)
    assert output.json['err_desc'] == 'invalid signature'

def test_get_user_from_api_query_string_error_missing_timestamp():
    signature = urllib.quote(
            base64.b64encode(
                hmac.new('1234',
                    'format=json&orig=coucou&algo=sha1', 
                    hashlib.sha1).digest()))
    output = get_app().get('/user?format=json&orig=coucou&algo=sha1&signature=%s' % signature, status=403)
    assert output.json['err_desc'] == 'missing/multiple timestamp field'

def test_get_user_from_api_query_string_error_missing_email():
    timestamp = datetime.datetime.utcnow().isoformat()[:19] + 'Z'
    query = 'format=json&orig=coucou&algo=sha1&timestamp=' + timestamp
    signature = urllib.quote(
            base64.b64encode(
                hmac.new('1234',
                    query, 
                    hashlib.sha1).digest()))
    output = get_app().get('/user?%s&signature=%s' % (query, signature), status=403)
    assert output.json['err_desc'] == 'missing email or NameID fields'

def test_get_user_from_api_query_string_error_success_sha1():
    timestamp = datetime.datetime.utcnow().isoformat()[:19] + 'Z'
    query = 'format=json&orig=coucou&algo=sha1&email=' + urllib.quote(user.email) + '&timestamp=' + timestamp
    signature = urllib.quote(
            base64.b64encode(
                hmac.new('1234',
                    query, 
                    hashlib.sha1).digest()))
    output = get_app().get('/user?%s&signature=%s' % (query, signature))
    assert output.json['user_display_name'] == u'Jean Darmette'

def test_get_user_from_api_query_string_error_invalid_signature_algo_mismatch():
    timestamp = datetime.datetime.utcnow().isoformat()[:19] + 'Z'
    query = 'format=json&orig=coucou&algo=sha256&email=' + urllib.quote(user.email) + '&timestamp=' + timestamp
    signature = urllib.quote(
            base64.b64encode(
                hmac.new('1234',
                    query, 
                    hashlib.sha1).digest()))
    output = get_app().get('/user?%s&signature=%s' % (query, signature), status=403)
    assert output.json['err_desc'] == 'invalid signature'

def test_get_user_from_api_query_string_error_success_sha256():
    timestamp = datetime.datetime.utcnow().isoformat()[:19] + 'Z'
    query = 'format=json&orig=coucou&algo=sha256&email=' + urllib.quote(user.email) + '&timestamp=' + timestamp
    signature = urllib.quote(
            base64.b64encode(
                hmac.new('1234',
                    query, 
                    hashlib.sha256).digest()))
    output = get_app().get('/user?%s&signature=%s' % (query, signature))
    assert output.json['user_display_name'] == u'Jean Darmette'
